#include <beagle/GP.hpp>
#include "include/SymbRegEvalOp.hpp"
#include "include/InitialData.h"

using namespace Beagle;

GP::System::Handle create_system() {
        // Build primitive set
        GP::PrimitiveSet::Handle lSet = new GP::PrimitiveSet;
        lSet->insert(new GP::Add);
        lSet->insert(new GP::Subtract);
        lSet->insert(new GP::Multiply);
        lSet->insert(new GP::Divide);
//        lSet->insert(new GP::Sin);
//        lSet->insert(new GP::Cos);
        lSet->insert(new GP::TokenT<Double>("x"));

        // Build a system
        return new GP::System(lSet);
}

int main(int argc, char *argv[]) {
    try {

        GP::System::Handle lSystem = create_system();
        // Create the initial data
        InitialData::Handle id = new InitialData(20);
        lSystem->addComponent(id);

        // Create the evaluation operator
        SymbRegEvalOp::Handle lEvalOp = new SymbRegEvalOp;

        // Create the standard GP evolver and vivarium
        GP::Evolver::Handle lEvolver = new GP::Evolver(lEvalOp);
        GP::Vivarium::Handle lVivarium = new GP::Vivarium;

        lEvolver->initialize(lSystem,argc,argv);
        lEvolver->evolve(lVivarium);
    }
    catch(Exception& inException) {
        inException.terminate();
    }
    return 0;
}
