use XML::XPath;
use GraphViz::XML;
use Getopt::Long;

GetOptions("infile=s" => \$infile,
		"treefile=s" => \$treefile,
		"gplotdata=s" => \$gplotdatafile,
		"gplotscript=s" => \$gplotscript,
		"graphfile=s" => \$graphfile
		);

my $xp = XML::XPath->new(filename=>$infile);

my @xx = split('\n',$xp->find('/Beagle/System/InitialData/X')->get_node(1)->string_value());
my @yy = split('\n',$xp->find('/Beagle/System/InitialData/Y')->get_node(1)->string_value());
my @final_x = ();
my @final_y = ();
for $l (@xx)
{
	if ($l =~ /[+-]?(\d+\.\d+|\d+\.|\.\d+|\d+)/)
	{
		push(@final_x,$l+0);
	}
}
for $l (@yy)
{
	if ($l =~ /[+-]?(\d+\.\d+|\d+\.|\.\d+|\d+)/)
	{
		push(@final_y,$l+0);
	}
}

open(GPLOTDATAFILE,">$gplotdatafile");
while (@final_x)
{
	print GPLOTDATAFILE pop(@final_x)."\t".pop(@final_y)."\n";
}
close(GPLOTDATAFILE);

my $nodeset = $xp->find('/Beagle/Vivarium/HallOfFame/Member/Individual/Genotype');
my $best_individual = $nodeset->get_node(1)->toString;
my $g = GraphViz::XML->new($best_individual);
open(TREEFILE,">$treefile");
print TREEFILE $g->as_ps;
close(TREEFILE);

open(GPLOTSCRIPT, ">$gplotscript");
print GPLOTSCRIPT "set terminal postscript color;\n";
print GPLOTSCRIPT "set output \"$graphfile\";\n";
my $fitness = $xp->find('/Beagle/Vivarium/HallOfFame/Member/Individual/Fitness')->get_node(1)->string_value();
print GPLOTSCRIPT "set title \"Fitness = " . $fitness . "\";\n";
print GPLOTSCRIPT "plot [-1:1] " . create_string($nodeset->get_node(1)->getChildNode(1)) . " with lines lw 3 lc rgbcolor \"blue\", ";
print GPLOTSCRIPT "\"$gplotdatafile\" with points ps 1 pt 3 lc rgbcolor \"black\" title \"Initial data\";\n";
close(GPLOTSCRIPT);


sub create_string {
	my $curnode = $_[0];
	if ($curnode->getName() eq "x") {return "x";}
	my $op = ""; my $type;
	my $opname = $curnode->getName();
	if ($opname eq "ADD") {$op = "+"; $type=2;}
	elsif ($opname eq "SUB") {$op = "-"; $type=2;}
	elsif ($opname eq "MUL") {$op = "*"; $type=2;}
	elsif ($opname eq "DIV") {$op = "/"; $type=2;}
	elsif ($opname eq "SIN") {$op = "sin"; $type=1;}
	elsif ($opname eq "COS") {$op = "cos"; $type=1;}
	
	if ($type == 2) {
		return '(' . create_string($curnode->getChildNode(1)).  $op . create_string($curnode->getChildNode(2)) . ')';
	}
	else
	{
		return $op . '(' . create_string($curnode->getChildNode(1)) . ')';
	}
}
